/*******************************************************************************
Copyright (C) Marvell International Ltd. and its affiliates

This software file (the "File") is owned and distributed by Marvell 
International Ltd. and/or its affiliates ("Marvell") under the following
alternative licensing terms.  Once you have made an election to distribute the
File under one of the following license alternatives, please (i) delete this
introductory statement regarding license alternatives, (ii) delete the two
license alternatives that you have not elected to use and (iii) preserve the
Marvell copyright notice above.

********************************************************************************
Marvell Commercial License Option

If you received this File from Marvell and you have entered into a commercial
license agreement (a "Commercial License") with Marvell, the File is licensed
to you under the terms of the applicable Commercial License.

********************************************************************************
Marvell GPL License Option

If you received this File from Marvell, you may opt to use, redistribute and/or 
modify this File in accordance with the terms and conditions of the General 
Public License Version 2, June 1991 (the "GPL License"), a copy of which is 
available along with the File in the license.txt file or by writing to the Free 
Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 or 
on the worldwide web at http://www.gnu.org/licenses/gpl.txt. 

THE FILE IS DISTRIBUTED AS-IS, WITHOUT WARRANTY OF ANY KIND, AND THE IMPLIED 
WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE ARE EXPRESSLY 
DISCLAIMED.  The GPL License provides additional details about this warranty 
disclaimer.
********************************************************************************
Marvell BSD License Option

If you received this File from Marvell, you may opt to use, redistribute and/or 
modify this File under the following licensing terms. 
Redistribution and use in source and binary forms, with or without modification, 
are permitted provided that the following conditions are met:

    *   Redistributions of source code must retain the above copyright notice,
	    this list of conditions and the following disclaimer. 

    *   Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution. 

    *   Neither the name of Marvell nor the names of its contributors may be 
        used to endorse or promote products derived from this software without 
        specific prior written permission. 
    
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR 
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#include "mvGpp.h"
#include "mvGppRegs.h"

/* defines  */       
#ifdef MV_DEBUG         
	#define DB(x)	x
#else                
	#define DB(x)    
#endif	             

//#define DEBUG
#if defined DEBUG
 #define TRACE(x)	x
#else
 #define TRACE(x)
#endif

static MV_VOID gppRegSet(MV_U32 group, MV_U32 regOffs,MV_U32 mask,MV_U32 value);

/*******************************************************************************
* mvGppOutEnablle - Enable a GPP (OUT) pin
*
* DESCRIPTION:
*
* INPUT:
*       group - GPP group number
*       mask  - 32bit mask value. Each set bit in the mask means that the type
*               of corresponding GPP will be set. Other GPPs are ignored.
*       value - 32bit value that describes GPP type per pin.
*
* OUTPUT:
*       None.
*
* EXAMPLE:
*       Set GPP8 to input and GPP15 to output.
*       mvGppOutEnablle(0, (GPP8 | GPP15), 
*                    ((MV_GPP_IN & GPP8) | (MV_GPP_OUT & GPP15)) );
*
* RETURN:
*       None.
*
*******************************************************************************/
MV_STATUS mvGppOutEnablle(MV_U32 group, MV_U32 mask, MV_U32 value)
{
	if (group >= MV_GPP_MAX_GROUP)
	{
		mvOsPrintf("mvGppOutEnablle: ERR. invalid group number \n");
		return MV_BAD_PARAM;
	}

	gppRegSet(group, GPP_DATA_OUT_EN_REG(group), mask, value);

	return MV_OK;

}

/*******************************************************************************
* mvGppBlinkEn - Set a GPP (IN) Pin list to blink every ~100ms
*
* DESCRIPTION:
*
* INPUT:
*       group - GPP group number
*       mask  - 32bit mask value. Each set bit in the mask means that the type
*               of corresponding GPP will be set. Other GPPs are ignored.
*       value - 32bit value that describes GPP blink per pin.
*
* OUTPUT:
*       None.
*
* EXAMPLE:
*       Set GPP8 to be static and GPP15 to be blinking.
*       mvGppBlinkEn(0, (GPP8 | GPP15), 
*                    ((MV_GPP_OUT_STATIC & GPP8) | (MV_GPP_OUT_BLINK & GPP15)) );
*
* RETURN:
*       None.
*
*******************************************************************************/
MV_STATUS mvGppBlinkEn(MV_U32 group, MV_U32 mask, MV_U32 value)
{
	if (group >= MV_GPP_MAX_GROUP)
	{
		DB(mvOsPrintf("mvGppBlinkEn: ERR. invalid group number \n"));
		return MV_BAD_PARAM;
	}

	gppRegSet(group, GPP_BLINK_EN_REG(group), mask, value);

	return MV_OK;

}
/*******************************************************************************
* mvGppPolaritySet - Set a GPP (IN) Pin list Polarity mode
*
* DESCRIPTION:
*
* INPUT:
*       group - GPP group number
*       mask  - 32bit mask value. Each set bit in the mask means that the type
*               of corresponding GPP will be set. Other GPPs are ignored.
*       value - 32bit value that describes GPP polarity per pin.
*
* OUTPUT:
*       None.
*
* EXAMPLE:
*       Set GPP8 to the actual pin value and GPP15 to be inverted.
*       mvGppPolaritySet(0, (GPP8 | GPP15), 
*                    ((MV_GPP_IN_ORIGIN & GPP8) | (MV_GPP_IN_INVERT & GPP15)) );
*
* RETURN:
*       None.
*
*******************************************************************************/
MV_STATUS mvGppPolaritySet(MV_U32 group, MV_U32 mask, MV_U32 value)
{
	if (group >= MV_GPP_MAX_GROUP)
	{
		DB(mvOsPrintf("mvGppPolaritySet: ERR. invalid group number \n"));
		return MV_BAD_PARAM;
	}

	gppRegSet(group, GPP_DATA_IN_POL_REG(group), mask, value);

	return MV_OK;

}



/*******************************************************************************
* mvGppValueGet - Get a GPP Pin list value.
*
* DESCRIPTION:
*       This function get GPP value.
*
* INPUT:
*       group - GPP group number
*       mask  - 32bit mask value. Each set bit in the mask means that the 
*               returned value is valid for it.
*
* OUTPUT:
*       None.
*
* EXAMPLE:
*       Get GPP8 and GPP15 value.
*       mvGppValueGet(0, (GPP8 | GPP15));
*
* RETURN:
*       32bit value that describes GPP activity mode per pin.
*
*******************************************************************************/
MV_U32 mvGppValueGet(MV_U32 group, MV_U32 mask)
{
	MV_U32 gppData;

	gppData = MV_REG_READ(GPP_DATA_IN_REG(group));

	gppData &= mask;

	return gppData;

}

/*******************************************************************************
* mvGppValueSet - Set a GPP Pin list value.
*
* DESCRIPTION:
*       This function set value for given GPP pin list.
*
* INPUT:
*       group - GPP group number
*       mask  - 32bit mask value. Each set bit in the mask means that the 
*               value of corresponding GPP will be set accordingly. Other GPP 
*               are not affected.
*       value - 32bit value that describes GPP value per pin.
*
* OUTPUT:
*       None.
*
* EXAMPLE:
*       Set GPP8 value of '0' and GPP15 value of '1'.
*       mvGppActiveSet(0, (GPP8 | GPP15), ((0 & GPP8) | (GPP15)) );
*
* RETURN:
*       None.
*
*******************************************************************************/
MV_STATUS mvGppValueSet (MV_U32 group, MV_U32 mask, MV_U32 value)
{
	MV_U32 outEnable;
	MV_U32 i;

	if (group >= MV_GPP_MAX_GROUP)
	{
		DB(mvOsPrintf("mvGppValueSet: Error invalid group number \n"));
		return MV_BAD_PARAM;
	}

	/* verify that the gpp pin is configured as output 		*/
	/* Note that in the register out enabled -> bit = '0'. 	*/
	outEnable = ~MV_REG_READ(GPP_DATA_OUT_EN_REG(group));

	for (i = 0 ; i < 32 ;i++)
	{
		if (((mask & (1 << i)) & (outEnable & (1 << i))) != (mask & (1 << i)))
		{
			mvOsPrintf("mvGppValueSet: Err. An attempt to set output "\
					   "value to GPP %d in input mode.\n", i);
			return MV_ERROR;
		}
	}

	gppRegSet(group, GPP_DATA_OUT_REG(group), mask, value);

	return MV_OK;

}
/*******************************************************************************
* gppRegSet - Set a specific GPP pin on a specific GPP register
*
* DESCRIPTION:
*       This function set a specific GPP pin on a specific GPP register
*
* INPUT:
*		regOffs - GPP Register offset 
*       group - GPP group number
*       mask  - 32bit mask value. Each set bit in the mask means that the 
*               value of corresponding GPP will be set accordingly. Other GPP 
*               are not affected.
*       value - 32bit value that describes GPP value per pin.
*
* OUTPUT:
*       None.
*
* EXAMPLE:
*       Set GPP8 value of '0' and GPP15 value of '1'.
*       mvGppActiveSet(0, (GPP8 | GPP15), ((0 & GPP8) | (1 & GPP15)) );
*
* RETURN:
*       None.
*
*******************************************************************************/
static MV_VOID gppRegSet (MV_U32 group, MV_U32 regOffs,MV_U32 mask,MV_U32 value)
{
	MV_U32 gppData;

	gppData = MV_REG_READ(regOffs);

	gppData &= ~mask;

	gppData |= (value & mask);

	MV_REG_WRITE(regOffs, gppData);
}

#if defined BUFFALO_SUPPORT_VIRTUALIZED_GPP
/*
	0	hdd_power
	1	hdd_power1	// newly added 2008/1/23
	2	usb_power
	3	usb_power2	// newly added 2008/2/25
	4	led_info				
	5	led_power				
	6	led_alarm				
	7	led_func	// newly added 2008/1/15
	8	led_reserve1	// newly added 2008/1/23
	9	led_reserve2	// newly added 2008/1/23
	10	led_reserve3	// newly added 2008/1/23
	11	led_reserve4	// newly added 2008/1/23
	12	fan_full				
	13	fan_lock				
	14	sw_init					
	15	sw_power				
	16	sw_raid					
	17	sw_auto_power				
	18	sw_func					
	19	fan_low					
	20	fan_high                                
*/
struct buffalo_gpp_config_st buffalo_gpp_config[]={
/*        0   1   2   3   4   5   6   7    8   9  10  11  12  13  14  15   16  17  18  19  20 */
	{
		// micon model
		{-1},
	},
	{
		// cube
		.hdd_power	=  2,
		.hdd_power1	= -1,
		.usb_power	= -1,
		.usb_power2	= -1,
		.usb_power3	= -1,
		.usb_power4	= -1,
		.led_info	=  3,
		.led_power	=  5,
		.led_alarm	=  6,
		.led_func	= -1,
		.led_reserve1	= -1,
		.led_wlan	= -1,
		.led_secure	= -1,
		.led_aoss	= -1,
		.fan_full	= -1,
		.fan_lock	=  7,
		.sw_init	= 10,
		.sw_power	= 12,
		.sw_raid	= -1,
		.sw_auto_power	= -1,
		.sw_func	= -1,
		.sw_aoss	= -1,
		.fan_low	= -1,
		.fan_high	= -1,
	},
	{
		// wtgl
		.hdd_power	=  1,
		.hdd_power1	= -1,
		.usb_power	=  9,
		.usb_power2	= -1,
		.usb_power3	= -1,
		.usb_power4	= -1,
		.led_info	=  3,
		.led_power	=  0,
		.led_alarm	=  2,
		.led_func	= -1,
		.led_reserve1	= -1,
		.led_wlan	= -1,
		.led_secure	= -1,
		.led_aoss	= -1,
		.fan_full	= 14,
		.fan_lock	=  6,
		.sw_init	=  7,
		.sw_power	=  8,
		.sw_raid	= -1,
		.sw_auto_power	= 10,
		.sw_func	= -1,
		.sw_aoss	= -1,
		.fan_low	= 17,
		.fan_high	= 14,
	},
	{
		// hgl
		.hdd_power	=  1,
		.hdd_power1	= -1,
		.usb_power	=  9,
		.usb_power2	= -1,
		.usb_power3	= -1,
		.usb_power4	= -1,
		.led_info	=  3,
		.led_power	=  0,
		.led_alarm	=  2,
		.led_func	= 17,
		.led_reserve1	= -1,
		.led_wlan	= -1,
		.led_secure	= -1,
		.led_aoss	= -1,
		.fan_full	= -1,
		.fan_lock	=  6,
		.sw_init	=  7,
		.sw_power	=  8,
		.sw_raid	= -1,
		.sw_auto_power	= 10,
		.sw_func	= 15,
		.sw_aoss	= -1,
		.fan_low	= -1,
		.fan_high	= -1,

	},
	{
		// wsgl
		.hdd_power	=  1,
		.hdd_power1	= 19,
		.usb_power	= 16,
		.usb_power2	= -1,
		.usb_power3	= -1,
		.usb_power4	= -1,
		.led_info	=  3,
		.led_power	= 14,
		.led_alarm	=  2,
		.led_func	=  9,
		.led_reserve1	=  0,
		.led_wlan	= -1,
		.led_secure	= -1,
		.led_aoss	= -1,
		.fan_full	= -1,
		.fan_lock	= -1,
		.sw_init	=  7,
		.sw_power	= 18,
		.sw_raid	= -1,
		.sw_auto_power	= 17,
		.sw_func	= 15,
		.sw_aoss	= -1,
		.fan_low	= -1,
		.fan_high	= -1,
	},
	{
		// whgl
		.hdd_power	=  1,
		.hdd_power1	= -1,
		.usb_power	=  9,
		.usb_power2	= 19,
		.usb_power3	= -1,
		.usb_power4	= -1,
		.led_info	=  3,
		.led_power	=  0,
		.led_alarm	=  2,
		.led_func	= 18,
		.led_reserve1	= -1,
		.led_wlan	= -1,
		.led_secure	= -1,
		.led_aoss	= -1,
		.fan_full	= -1,
		.fan_lock	=  6,
		.sw_init	=  7,
		.sw_power	= 10,
		.sw_raid	= -1,
		.sw_auto_power	= 22,
		.sw_func	=  8,
		.sw_aoss	= -1,
		.fan_low	= 17,
		.fan_high	= 14,
	},
	{
		// wwn/r1
		.hdd_power	= 12,
		.hdd_power1	= 11,
		.usb_power	= 10,
		.usb_power2	=  9,
		.usb_power3	= 15,
		.usb_power4	=  1,
		.led_info	= 22,
		.led_power	= 16,
		.led_alarm	= 23,
		.led_func	= 17,
		.led_reserve1	= -1,
		.led_wlan	= 14,
		.led_secure	= 13,
		.led_aoss	=  8,
		.fan_full	= -1,
		.fan_lock	= 26,
		.sw_init	= -1,
		.sw_power	= 20,
		.sw_raid	= -1,
		.sw_auto_power	= 19,
		.sw_func	= 21,
		.sw_aoss	= 18,
		.fan_low	= 25,
		.fan_high	= 24,
	},
	{
		// CL
		.hdd_power	=  1,
		.hdd_power1	= -1,
		.usb_power	=  9,
		.usb_power2	= -1,
		.usb_power3	= -1,
		.usb_power4	= -1,
		.led_info	=  3,
		.led_info_group	=  0x01,
		.led_power	=  0,
		.led_power_group=  0x02,
		.led_alarm	=  2,
		.led_alarm_group=  0x00,
		.led_func	= 17,
		.led_func_group	= -1,
		.led_eth	=  0,
		.led_eth_group	= -1,
		.led_reserve1	= -1,
		.led_wlan	= -1,
		.led_secure	= -1,
		.led_aoss	= -1,
		.fan_full	= -1,
		.fan_lock	=  6,
		.sw_init	=  7,
		.sw_power	=  8,
		.sw_raid	= -1,
		.sw_auto_power	= 10,
		.sw_func	= 15,
		.sw_aoss	= -1,
		.fan_low	= 16,
		.fan_high	= 14,
	},
};

struct buffalo_gpp_config_st buffalo_gpp_config_atribute[] = {
	{
		// micon model
		{0},
	},
	{
		// cube
		{0},
	},
	{
		// wtgl
		{0},
	},
	{
		// hgl
		{0},
	},
	{
		// wsgl
		.led_power	= ATRIBUTE_REVERSE,
	},
	{
		// whgl
		{0},
	},
	{
		// wwn
		{0},
	},
	{
		// cl
		{0},
	},
};

int buffalo_board_id_to_gpp_config[]={
					N_A,	//0x00
					N_A,	//0x01
					N_A,	//0x02
					N_A,	//0x03
					N_A,	//0x04
					N_A,	//0x05
					N_A,	//0x06
					N_A,	//0x07
					N_A,	//0x08
					N_A,	//0x09
					N_A,	//0x0a
					N_A,	//0x0b
					N_A,	//0x0c
					N_A,	//0x0d
					N_A,	//0x0e
					N_A,	//0x0f
					N_A,	//0x10
					N_A,	//0x11
					0,	//0x12	LS-GL
					N_A,	//0x13
					0,	//0x14	TS-HTGL
					N_A,	//0x15
					N_A,	//0x16
					0,	//0x17	HS-WDHTGL
					2,	//0x18	LS-WTGL
					1,	//0x19  LS-WGL
					4,	//0x1a  LS-WSGL
					3,	//0x1b	LS-HGL
					5,	//0x1c	LS-WHGL
					6,	//0x1d	LS-WWN/R1
					7,	//0x1e	LS-CL/CHL
};
#endif


MV_VOID
buffalo_gpp_init(void)
{
	MV_U32 CPU_ID = BuffaloGetCpuId();

	// initialize of hdd power.
	mvGppOutEnablle(0, (BIT_HDD_POWER),
				((MV_GPP_OUT & BIT_HDD_POWER)));
	mvGppPolaritySet(0, (BIT_HDD_POWER),
				((MV_GPP_OUT_DIS & BIT_HDD_POWER)));

	if(PIN_NUM_USB_POWER != 255)
	{
		mvGppOutEnablle(0, (BIT_USB_POWER),
				((MV_GPP_OUT & BIT_USB_POWER)));
		mvGppPolaritySet(0, (BIT_USB_POWER),
					((MV_GPP_OUT_DIS & BIT_USB_POWER)));
	}
	if(PIN_NUM_USB_POWER2 != 255)
	{
		mvGppOutEnablle(0, (BIT_USB_POWER2),
				((MV_GPP_OUT & BIT_USB_POWER2)));
		mvGppPolaritySet(0, (BIT_USB_POWER2),
					((MV_GPP_OUT_DIS & BIT_USB_POWER2)));
	}
	if(PIN_NUM_USB_POWER3 != 255)
	{
		mvGppOutEnablle(0, (BIT_USB_POWER3),
				((MV_GPP_OUT & BIT_USB_POWER3)));
		mvGppPolaritySet(0, (BIT_USB_POWER3),
					((MV_GPP_OUT_DIS & BIT_USB_POWER3)));
	}
	if(PIN_NUM_USB_POWER4 != 255)
	{
		mvGppOutEnablle(0, (BIT_USB_POWER4),
				((MV_GPP_OUT & BIT_USB_POWER4)));
		mvGppPolaritySet(0, (BIT_USB_POWER4),
					((MV_GPP_OUT_DIS & BIT_USB_POWER4)));
	}

	if(PIN_NUM_FAN_LOCK != 255)
	{
		mvGppOutEnablle(0, (BIT_FAN_LOCK),
				((MV_GPP_IN & BIT_FAN_LOCK)));
		mvGppPolaritySet(0, (BIT_FAN_LOCK),
				((MV_GPP_IN_ORIGIN & BIT_FAN_LOCK)));
	}

	// ---> 2007/11/02 Y.H.
	if(PIN_NUM_FAN_LOW != 255)
	{
		mvGppOutEnablle(0, (BIT_FAN_LOW),
				((MV_GPP_OUT & BIT_FAN_LOW)));
		mvGppPolaritySet(0, (BIT_FAN_LOW),
				((MV_GPP_OUT_DIS & BIT_FAN_LOW)));
	}

	if(PIN_NUM_FAN_HIGH != 255)
	{
		mvGppOutEnablle(0, (BIT_FAN_HIGH),
				((MV_GPP_OUT & BIT_FAN_HIGH)));
		mvGppPolaritySet(0, (BIT_FAN_HIGH),
				((MV_GPP_OUT_DIS & BIT_FAN_HIGH)));
	}
	// <--- 2007/11/02 Y.H.

	// initialize of sw around.
	if(PIN_NUM_SW_POWER != 255)
	{
		mvGppOutEnablle(0, BIT_SW_POWER, 
				(MV_GPP_IN & BIT_SW_POWER));

		mvGppPolaritySet(0, BIT_SW_POWER,
				 (MV_GPP_IN_INVERT & BIT_SW_POWER));
	}

	if(PIN_NUM_SW_INIT != 255)
	{
		mvGppOutEnablle(0, BIT_SW_INIT, 
				 (MV_GPP_IN & BIT_SW_INIT));

		mvGppPolaritySet(0, BIT_SW_INIT,
				(MV_GPP_IN_INVERT & BIT_SW_INIT));
	}

	if(PIN_NUM_SW_AUTO_POWER != 255)
	{
		mvGppOutEnablle(0, BIT_SW_AUTO_POWER,
				(MV_GPP_IN & BIT_SW_AUTO_POWER));
		mvGppPolaritySet(0, (BIT_SW_AUTO_POWER),
				(MV_GPP_IN_INVERT & BIT_SW_AUTO_POWER));
	}

	if(PIN_NUM_SW_FUNC != 255)
	{
		mvGppOutEnablle(0, BIT_SW_FUNC,
				(MV_GPP_IN & BIT_SW_FUNC));
		mvGppPolaritySet(0, (BIT_SW_FUNC),
				(MV_GPP_IN_INVERT & BIT_SW_FUNC));
	}

	// initialize of led around.
	if(PIN_NUM_LED_INFO != 255)
	{
		if(0x5281 == CPU_ID)
		{
			MV_REG_BIT_SET(GPP_DATA_OUT_REG(0), BIT_LED_INFO);
			MV_REG_BIT_RESET(GPP_DATA_OUT_EN_REG(0), BIT_LED_INFO);
		}
		else
		{
			mvGppOutEnablle(0, BIT_LED_INFO,
				(MV_GPP_OUT_DIS & BIT_LED_INFO));
		}
		mvGppBlinkEn(0, BIT_LED_INFO,
				(MV_GPP_OUT_STATIC & BIT_LED_INFO));
	}

	if(PIN_NUM_LED_ALARM != 255)
	{
		if(0x5281 == CPU_ID)
		{
			MV_REG_BIT_SET(GPP_DATA_OUT_REG(0), BIT_LED_ALARM);
			MV_REG_BIT_RESET(GPP_DATA_OUT_EN_REG(0), BIT_LED_ALARM);
		}
		else
		{
			mvGppOutEnablle(0, BIT_LED_ALARM,
				 (MV_GPP_OUT_DIS & BIT_LED_ALARM));
		}
		mvGppBlinkEn(0, BIT_LED_ALARM,
				 (MV_GPP_OUT_STATIC & BIT_LED_ALARM));
	}

	if(PIN_NUM_LED_POWER != 255)
	{
		if(0x5281 == CPU_ID)
		{
			MV_REG_BIT_SET(GPP_DATA_OUT_REG(0), BIT_LED_POWER);
			MV_REG_BIT_RESET(GPP_DATA_OUT_EN_REG(0), BIT_LED_POWER);
		}
		else
		{
			mvGppOutEnablle(0, BIT_LED_POWER,
				 (MV_GPP_OUT_DIS & BIT_LED_POWER));
		}
		mvGppBlinkEn(0, BIT_LED_POWER,
			 (MV_GPP_OUT_STATIC & BIT_LED_POWER));

		if(ATRIBUTE_LED_POWER == ATRIBUTE_REVERSE)
		{
			mvGppPolaritySet(0, (BIT_SW_FUNC),
					(MV_GPP_IN_INVERT & BIT_SW_FUNC));
		}
	}

	if(PIN_NUM_LED_FUNC != 255)
	{
		if(0x5281 == CPU_ID)
		{
			MV_REG_BIT_SET(GPP_DATA_OUT_REG(0), BIT_LED_FUNC);
			MV_REG_BIT_RESET(GPP_DATA_OUT_EN_REG(0), BIT_LED_POWER);
		}
		else
		{
			mvGppOutEnablle(0, BIT_LED_FUNC,
				 (MV_GPP_OUT_DIS & BIT_LED_FUNC));
		}
		mvGppBlinkEn(0, BIT_LED_FUNC,
			 (MV_GPP_OUT_STATIC & BIT_LED_FUNC));
	}

	if(PIN_NUM_LED_RESERVE1 != 255)
	{
		if(0x5281 == CPU_ID)
		{
			MV_REG_BIT_SET(GPP_DATA_OUT_REG(0), BIT_LED_RESERVE1);
			MV_REG_BIT_RESET(GPP_DATA_OUT_EN_REG(0), BIT_LED_RESERVE1);
		}
		else
		{
			mvGppOutEnablle(0, BIT_LED_RESERVE1,
				 (MV_GPP_OUT_DIS & BIT_LED_RESERVE1));
		}
		mvGppBlinkEn(0, BIT_LED_RESERVE1,
				 (MV_GPP_OUT_STATIC & BIT_LED_RESERVE1));
		if(ATRIBUTE_LED_RESERVE1 == ATRIBUTE_REVERSE)
		{
			mvGppPolaritySet(0, (BIT_LED_RESERVE1),
					(MV_GPP_IN_INVERT & BIT_LED_RESERVE1));
		}
	}

	if(PIN_NUM_LED_WLAN != 255)
	{
		if(0x5281 == CPU_ID)
		{
			MV_REG_BIT_SET(GPP_DATA_OUT_REG(0), BIT_LED_WLAN);
			MV_REG_BIT_RESET(GPP_DATA_OUT_EN_REG(0), BIT_LED_WLAN);
		}
		else
		{
			mvGppOutEnablle(0, BIT_LED_WLAN,
				 (MV_GPP_OUT_DIS & BIT_LED_WLAN));
		}
		mvGppBlinkEn(0, BIT_LED_WLAN,
				 (MV_GPP_OUT_STATIC & BIT_LED_WLAN));
	}
	if(PIN_NUM_LED_SECURE != 255)
	{
		if(0x5281 == CPU_ID)
		{
			MV_REG_BIT_SET(GPP_DATA_OUT_REG(0), BIT_LED_SECURE);
			MV_REG_BIT_RESET(GPP_DATA_OUT_EN_REG(0), BIT_LED_SECURE);
		}
		else
		{
			mvGppOutEnablle(0, BIT_LED_SECURE,
				 (MV_GPP_OUT_DIS & BIT_LED_SECURE));
		}
		mvGppBlinkEn(0, BIT_LED_SECURE,
				 (MV_GPP_OUT_STATIC & BIT_LED_SECURE));
	}
	if(PIN_NUM_LED_AOSS != 255)
	{
		if(0x5281 == CPU_ID)
		{
			MV_REG_BIT_SET(GPP_DATA_OUT_REG(0), BIT_LED_AOSS);
			MV_REG_BIT_RESET(GPP_DATA_OUT_EN_REG(0), BIT_LED_AOSS);
		}
		else
		{
			mvGppOutEnablle(0, BIT_LED_AOSS,
				 (MV_GPP_OUT_DIS & BIT_LED_AOSS));
		}
		mvGppBlinkEn(0, BIT_LED_AOSS,
				 (MV_GPP_OUT_STATIC & BIT_LED_AOSS));
	}
	// fan power is interlocked to hdd power.
}

MV_VOID
buffalo_gpp_led_control(MV_U32 target_led, MV_U32 status_flag)
{
//	printf("target_led = 0x%08x\n""status_flag = 0x%08x\n", target_led, status_flag);
	MV_U32 CPU_ID = BuffaloGetCpuId();

	switch(status_flag){
	case BUFFALO_LED_FLAG_ON:
		if(0x5281 == CPU_ID)
		{
			MV_REG_BIT_RESET(GPP_DATA_OUT_REG(0), target_led);
			MV_REG_BIT_RESET(GPP_DATA_OUT_EN_REG(0), target_led);
		}
		else
		{
			mvGppOutEnablle(0, target_led, (MV_GPP_OUT_EN & target_led));
		}
		mvGppBlinkEn(0, target_led, (MV_GPP_OUT_STATIC & target_led));
		break;

	case BUFFALO_LED_FLAG_OFF:
		if(0x5281 == CPU_ID)
		{
			MV_REG_BIT_SET(GPP_DATA_OUT_REG(0), target_led);
			MV_REG_BIT_RESET(GPP_DATA_OUT_EN_REG(0), target_led);
		}
		else
		{
			mvGppOutEnablle(0, target_led, (MV_GPP_OUT_DIS & target_led));
		}
		mvGppBlinkEn(0, target_led, (MV_GPP_OUT_STATIC & target_led));
		break;

	case BUFFALO_LED_FLAG_BLINK_ON:
		if(0x5281 == CPU_ID)
		{
			MV_REG_BIT_RESET(GPP_DATA_OUT_REG(0), target_led);
			MV_REG_BIT_RESET(GPP_DATA_OUT_EN_REG(0), target_led);
		}
		else
		{
			mvGppOutEnablle(0, target_led, (MV_GPP_OUT_EN & target_led));
		}
		mvGppBlinkEn(0, target_led, (MV_GPP_OUT_BLINK & target_led));
		break;
	default:
		break;
	}
}

MV_VOID
buffalo_gpp_hdd_power_control(MV_U32 status_flag)
{
	TRACE(printf("%s > BIT_HDD_POWER=0x%04x\n", __FUNCTION__, BIT_HDD_POWER));
	TRACE(printf("%s > status_flag = 0x%04x\n", __FUNCTION__, status_flag));

	switch(status_flag){
	case BUFFALO_HDD_POWER_ON:
		MV_REG_BIT_SET(GPP_DATA_OUT_REG(0), BIT_HDD_POWER);
		if(PIN_NUM_HDD_POWER1 != 255)
			MV_REG_BIT_SET(GPP_DATA_OUT_REG(0), BIT_HDD_POWER1);
		break;
	case BUFFALO_HDD_POWER_OFF:
		MV_REG_BIT_RESET(GPP_DATA_OUT_REG(0), BIT_HDD_POWER);
		if(PIN_NUM_HDD_POWER1 != 255)
			MV_REG_BIT_RESET(GPP_DATA_OUT_REG(0), BIT_HDD_POWER1);
		break;
	default:
		break;
	}
	TRACE(printf("%s > GPP_DATA_OUT_REG(0) & BIT_HDD_POWER = %d\n", 
			__FUNCTION__, MV_REG_READ(GPP_DATA_OUT_REG(0)) & BIT_HDD_POWER));
	TRACE(printf("%s > GPP_DATA_OUT_EN_REG(0) & BIT_HDD_POWER = %d\n",
			__FUNCTION__, MV_REG_READ(GPP_DATA_OUT_REG(0)) & BIT_HDD_POWER));
}

MV_VOID
buffalo_gpp_usb_power_control(MV_U32 status_flag)
{
	TRACE(printf("%s > BIT_USB_POWER=0x%04x\n", __FUNCTION__, BIT_USB_POWER));
	TRACE(printf("%s > status_flag = 0x%04x\n", __FUNCTION__, status_flag));
	TRACE(printf("%s > 0x10000 = 0x%08x\n", __FUNCTION__, MV_REG_READ(0x10000)));
	TRACE(printf("%s > 0x10004 = 0x%08x\n", __FUNCTION__, MV_REG_READ(0x10004)));
	TRACE(printf("%s > 0x10050 = 0x%08x\n", __FUNCTION__, MV_REG_READ(0x10050)));

	if(PIN_NUM_USB_POWER == -1)
		return ;
	
	switch(status_flag){
	case BUFFALO_USB_POWER_ON:
		MV_REG_BIT_SET(GPP_DATA_OUT_REG(0), BIT_USB_POWER);
		if(PIN_NUM_USB_POWER2 != 255)
			MV_REG_BIT_SET(GPP_DATA_OUT_REG(0), BIT_USB_POWER2);
		if(PIN_NUM_USB_POWER3 != 255)
			MV_REG_BIT_SET(GPP_DATA_OUT_REG(0), BIT_USB_POWER3);
		if(PIN_NUM_USB_POWER4 != 255)
			MV_REG_BIT_SET(GPP_DATA_OUT_REG(0), BIT_USB_POWER4);
		break;
	case BUFFALO_USB_POWER_OFF:
		MV_REG_BIT_RESET(GPP_DATA_OUT_REG(0), BIT_USB_POWER);
		if(PIN_NUM_USB_POWER2 != 255)
			MV_REG_BIT_RESET(GPP_DATA_OUT_REG(0), BIT_USB_POWER2);
		if(PIN_NUM_USB_POWER3 != 255)
			MV_REG_BIT_RESET(GPP_DATA_OUT_REG(0), BIT_USB_POWER3);
		if(PIN_NUM_USB_POWER4 != 255)
			MV_REG_BIT_RESET(GPP_DATA_OUT_REG(0), BIT_USB_POWER4);
		break;
	default:
		break;
	}
	TRACE(printf("%s > GPP_DATA_OUT_REG(0) & BIT_USB_POWER = %d\n", 
			__FUNCTION__, MV_REG_READ(GPP_DATA_OUT_REG(0)) & BIT_USB_POWER));
	TRACE(printf("%s > GPP_DATA_OUT_EN_REG(0) & BIT_USB_POWER = %d\n",
			__FUNCTION__, MV_REG_READ(GPP_DATA_OUT_REG(0)) & BIT_USB_POWER));
}

// ---> 2007/11/02 Y.H.
MV_VOID
buffalo_gpp_fan_control(MV_U32 status_flag)
{
	if(PIN_NUM_FAN_LOW == -1 || PIN_NUM_FAN_HIGH == -1)
		return ;

	switch(status_flag){
	case BUFFALO_FAN_CONTROL_STOP:
		MV_REG_BIT_SET(GPP_DATA_OUT_REG(0), BIT_FAN_LOW);
		MV_REG_BIT_SET(GPP_DATA_OUT_REG(0), BIT_FAN_HIGH);
		break;
	case BUFFALO_FAN_CONTROL_SLOW:
		MV_REG_BIT_SET(GPP_DATA_OUT_REG(0), BIT_FAN_LOW);
		MV_REG_BIT_RESET(GPP_DATA_OUT_REG(0), BIT_FAN_HIGH);
		break;
	case BUFFALO_FAN_CONTROL_FAST:
		MV_REG_BIT_RESET(GPP_DATA_OUT_REG(0), BIT_FAN_LOW);
		MV_REG_BIT_SET(GPP_DATA_OUT_REG(0), BIT_FAN_HIGH);
		break;
	case BUFFALO_FAN_CONTROL_FULL:
		MV_REG_BIT_RESET(GPP_DATA_OUT_REG(0), BIT_FAN_LOW);
		MV_REG_BIT_RESET(GPP_DATA_OUT_REG(0), BIT_FAN_HIGH);
		break;
	default:
		break;
	}
}
// <--- 2007/11/02 Y.H.
